# -*- coding: utf-8 -*-
"""
Manage snap packages via Salt

Backported from https://github.com/saltstack/salt/pull/56953 and added
the --classic argument

:depends: snapd for distribution
"""
import logging

import salt.utils.environment
import salt.utils.path
import salt.utils.yamlloader as yamlloader

log = logging.getLogger(__name__)

SNAP_BINARY_NAME = "snap"

__virtualname__ = "snap"


def __virtual__():
    if salt.utils.path.which(SNAP_BINARY_NAME):
        return __virtualname__

    return (
        False,
        'The snap execution module cannot be loaded: the "%s" binary is not in the path.'
        % SNAP_BINARY_NAME,
    )


def _call_snap(args, **kwargs):
    """
    Call snap.
    """
    params = {
        "output_loglevel": "trace",
        "python_shell": False,
        "env": salt.utils.environment.get_module_environment(globals()),
    }
    params.update(kwargs)
    cmd = [SNAP_BINARY_NAME]
    cmd.extend(args)
    return __salt__["cmd.run_all"](cmd, **params)


def install(pkg, channel=None, refresh=False, classic=False):
    """
    Install the specified snap package from the specified channel.
    Returns a dictionary of "result" and "output".

    pkg
        The snap package name

    channel
        Optional. The snap channel to install from, eg "beta"

    refresh : False
        If True, use "snap refresh" instead of "snap install".
        This allows changing the channel of a previously installed package.

    classic : False
        Install a classic snap package if True
    """
    args = []
    ret = {"result": None, "output": ""}

    if refresh:
        cmd = "refresh"
    else:
        cmd = "install"

    if channel:
        args.append("--channel=" + channel)

    if classic:
        args.append("--classic")

    out = _call_snap([cmd, pkg] + args, ignore_retcode=True)
    ret["output"] = ("\n".join([out["stderr"], out["stdout"]])).strip()
    ret["result"] = out["retcode"] == 0

    return ret


def is_installed(pkg):
    """
    Returns True if there is any version of the specified package installed.

    pkg
        The package name
    """
    return bool(versions_installed(pkg))


def remove(pkg):
    """
    Remove the specified snap package. Returns a dictionary of "result" and "output".

    pkg
        The package name
    """
    ret = {"result": None, "output": ""}
    out = _call_snap(["remove", pkg], ignore_retcode=True)
    ret["output"] = ("\n".join([out["stderr"], out["stdout"]])).strip()
    ret["result"] = out["retcode"] == 0
    return ret


def versions_installed(pkg):
    """
    Query which version(s) of the specified snap package are installed.
    Returns a list of 0 or more dictionaries.

    pkg
        The package name
    """

    out = _call_snap(["info", pkg], ignore_retcode=True)
    if out["retcode"] != 0:
        if out["stderr"]:
            log.debug(
                "Snap info %s returned error. Stderr follows: \n%s", pkg, out["stderr"]
            )
        return []

    info = yamlloader.load(out["stdout"])
    if "installed" not in info:
        return []

    ret = [{
        "name": info["name"],
        "tracking": info["tracking"],
        "publisher": info["publisher"],
        "rev": info["installed"].split()[1].strip("()"),
        "version": info["installed"].split()[0],
    }]

    return ret
