# -*- coding: utf-8 -*-
"""
Management of snap packages
===============================================

Backported from https://github.com/saltstack/salt/pull/56953 and added
the --classic argument
"""
import logging

import salt.utils.path

log = logging.getLogger(__name__)

SNAP_BINARY_NAME = "snap"

__virtualname__ = "snap"


def __virtual__():
    if salt.utils.path.which(SNAP_BINARY_NAME):
        return __virtualname__

    return (
        False,
        'The snap execution module cannot be loaded: the "%s" binary is not in the path.'
        % SNAP_BINARY_NAME,
    )


def installed(name, channel=None, classic=False):
    """
    Ensure that the named snap package is installed

    name
        The snap package

    channel
        Optional. The channel to install the package from.

    classic
        Optional. Install a classic snap package if True
    """
    ret = {"name": name, "changes": {}, "result": None, "comment": ""}

    old = __salt__["snap.versions_installed"](name)
    if not old:
        if __opts__["test"]:
            ret["comment"] = 'Package "{0}" would have been installed'.format(name)
            ret["changes"]["new"] = name
            ret["changes"]["old"] = None
            ret["result"] = None
            return ret

        install = __salt__["snap.install"](name, channel=channel, classic=classic)
        if install["result"]:
            ret["comment"] = 'Package "{0}" was installed'.format(name)
            ret["changes"]["new"] = name
            ret["changes"]["old"] = None
            ret["result"] = True
            return ret

        ret["comment"] = 'Package "{0}" failed to install:\n{1}'.format(
            name, install["output"]
        )
        ret["result"] = False
        return ret

    # Currently snap always returns only one line?
    old_channel = old[0]["tracking"]
    if old_channel != channel and channel is not None:
        if __opts__["test"]:
            ret[
                "comment"
            ] = 'Package "{0}" would have been switched to channel {1}'.format(
                name, channel
            )
            ret["changes"]["old_channel"] = old_channel
            ret["changes"]["new_channel"] = channel
            ret["result"] = None
            return ret

        refresh = __salt__["snap.install"](
            name, channel=channel, refresh=True, classic=classic
        )
        if refresh["result"]:
            ret["comment"] = 'Package "{0}" was switched to channel {1}'.format(
                name, channel
            )
            ret["changes"]["old_channel"] = old_channel
            ret["changes"]["new_channel"] = channel
            ret["result"] = True
            return ret

        ret["comment"] = 'Failed to switch Package "{0}" to channel {1}:\n{2}'.format(
            name, channel, refresh["output"]
        )
        ret["result"] = False
        return ret

    ret["comment"] = 'Package "{0}" is already installed'.format(name)
    if __opts__["test"]:
        ret["result"] = None
        return ret

    ret["result"] = True
    return ret


def removed(name):
    """
    Ensure that the named snap package is not installed

    name
        The snap package
    """

    ret = {"name": name, "changes": {}, "result": None, "comment": ""}

    old = __salt__["snap.versions_installed"](name)
    if not old:
        ret["comment"] = "Package {0} is not installed".format(name)
        ret["result"] = True
        return ret

    if __opts__["test"]:
        ret["comment"] = "Package {0} would have been removed".format(name)
        ret["result"] = None
        ret["changes"]["old"] = old[0]["version"]
        ret["changes"]["new"] = None
        return ret

    remove = __salt__["snap.remove"](name)
    ret["comment"] = "Package {0} removed".format(name)
    ret["result"] = True
    ret["changes"]["old"] = old[0]["version"]
    ret["changes"]["new"] = None
    return ret
